// Filename: Admin.cpp
// Description: Definition of Admin class

#include "Admin.h"

// Admin constructor
Admin::Admin(Socket * socket)
{
  socket_object=socket;
}

// The definition for AddRoom() member function
int Admin::AddRoom (char room[], char cap[])
{
    FileSys filesystem_object;
    char str[25];
    
    filesystem_object.OpenFile("rooms/listing.dat");
    sprintf(str,"%s %s",room, cap);
    filesystem_object.WriteFile(str,0,0);
    
    socket_object->Write("100 OK Successful addition of ");
    socket_object->Write(room); 
    socket_object->Write(" \n");
    
    filesystem_object.CloseFile();
    return 0;
}

// The definition for DeleteRoom() member function
int Admin::DeleteRoom (char room[])
{
    FileSys filesystem_object;
    int ret;

    filesystem_object.OpenFile("rooms/listing.dat");
    ret=filesystem_object.DeleteRegistry(room);
   
    if (ret==0)
      {
      // Send error response
      socket_object->Write("ERROR: Invalid Room. \n");
      }

    else
      {
      // Send response
      socket_object->Write("100 OK Successful deletion of ");   
      socket_object->Write(room);
      socket_object->Write(" \n");
      }

    filesystem_object.CloseFile();
    return 0;
}

// The definition for delete() member function
int Admin::Delete (char username[])
{
  // Local variables
  FileSys filesystem_object;
  int ret;
  
  // Don't allow for the root user to be deleted
  if (strcmp(username,"root")==0)
  {
    socket_object->Write("Error. Can not delete root. \n");
    return 0;
  }

  filesystem_object.OpenFile("registry.dat");
  ret=filesystem_object.DeleteRegistry(username);
      
  if (ret==0)
    {
      // Send error response
      socket_object->Write("308 Invalid Username. \n");
    }

  else
    {
    // Send response
    socket_object->Write("100 OK Successful deletion of ");   
    socket_object->Write(username);
    socket_object->Write(" \n");
    }

  filesystem_object.CloseFile();
  return 0;
}

// Defintion of add() member function
int Admin::add(char username[], char passwd[])
{
  // Local variables
  int status;

  status=internal_add(username,passwd);
  // If there was an error in the command, send appropriate response
  if (status==-1)  
    {
    socket_object->Write("308 Invalid Username. \n");
    return -1;
    } 

  else
    {
    // Else username was found
    socket_object->Write("100 OK Successful addition of ");
    socket_object->Write(username); 
    socket_object->Write(" \n");
    }

  return 0;
}

// Defintion of changepasswd() member function
int Admin::changepasswd(char username[], char passwd[])
{
  // Local variables
  int status;

  status=internal_changepasswd(username,passwd);

  // If there was an error in the command, send appropriate response
  if (status==-1)  
    socket_object->Write("310 Username does not exist. \n");
  else
  // Else username was found
    socket_object->Write("100 OK Password changed. \n");
 
  return 0;
}

// Defintion of internal_add() member function
int Admin::internal_add (char username[], char passwd[])
{
  // Local variables
  FileSys filesystem_object;
  char filecontents[BLOCKSIZE+1];
  char newentry[BLOCKSIZE+1];
  char name[9];
  char encryptedpasswd[17];
  int line;

  // Open contents
  filesystem_object.OpenFile("registry.dat");  
 
  // Look for duplicate username in registry.dat file
  for (line=1;;line++)
  {
    strcpy(filecontents,filesystem_object.ReadFile(line));

    // Exit for loop at end of list
    if (filecontents[0]=='\0')
      break;
    
    // Get username field
    strcpy(name,strtok(filecontents," "));

    // If have a duplicate username, send error response & exit for loop
    if (strcmp(name,username)==0)
    {
      // Close contents
      filesystem_object.CloseFile();  

      // Exit function
      return -1;
    }
  }

  // Encrypt password
  strcpy(encryptedpasswd,passwd); //=crypt(passwd,"69");

  // Construct new entry
  strcpy(newentry,username);
  strcat(newentry," ");
  strcat(newentry,encryptedpasswd);
  // Write new entry
  filesystem_object.WriteFile(newentry,0,0);

  // Close file
  filesystem_object.CloseFile();

  // Create user directory
  mkdir(username,02750);
  return 0;
}

// Defintion of internal_changepasswd() member function
int Admin::internal_changepasswd(char username[], char newpasswd[])
{
  // Local variables
  FileSys filesystem_object;
  char filecontents[BLOCKSIZE+1];
  char newentry[BLOCKSIZE+1];
  char name[9];
  char encryptedpasswd[17];
  int line;
  int totallines;

  // Initialization
  newentry[0]='\0';

  // Open contents
  filesystem_object.OpenFile("registry.dat");  
 
  // Figure out total number of lines in registry.dat
  for (totallines=1;;totallines++)
  {
    strcpy(filecontents,filesystem_object.ReadFile(totallines));

    // Exit for loop at end of list
    if (filecontents[0]=='\0')
      break;
  }

  // Compensate for extra line count
  totallines--;
   
  // Find matching username in registry.dat file
  for (line=1;;line++)
  {
    strcpy(filecontents,filesystem_object.ReadFile(line));

    // Exit for loop at end of list and report error
    if (filecontents[0]=='\0')
      return -1;
    
    // Get username field
    strcpy(name,strtok(filecontents," "));

    // If have a valid username , exit for loop
    if (strcmp(name,username)==0)
      break;
  }

  // Encrypt password
  strcpy(encryptedpasswd,newpasswd); //=crypt(newpasswd,"69");

  // Construct new entry
  strcpy(newentry,username);
  strcat(newentry," ");
  strcat(newentry,encryptedpasswd);

  // Write new entry
  filesystem_object.WriteFile(newentry,line,totallines);

  // Close file
  filesystem_object.CloseFile();

  return 0;
}
