#ifndef FILESYS_H
#define FILESYS_H

#include <sys/types.h>
#include <sys/stat.h> 
#include <fcntl.h>    
#include <unistd.h>   
#include <string.h>  
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#define OK              0
#define YES             0
#define NO              1 
#define ERROR          -1
#define BLOCKED        -2
#define BLOCKSIZE    4096

class FileSys
{
   public:
      int OpenFile (char *file);
      int OpenEventFile (char *file);
      int OpenUserInfoFile (char *file);
      void DeleteEvent (int linenum);
      void DeleteRoomEvent (char *event);
      void DeleteUserInfo (int linenum);
      int DeleteNotification (char *notification);
      int DeleteRegistry (char *username);
      char *ReadFile (int linenum);
      void WriteFile (char *writetxt, int linenum, int lastline);       
      void CloseFile (void);
      char *SearchRoomListing(int, int, int, int, int, int);      
      int DoesUserExist(char *);
      int DoesRoomExist(char *, int, int, int, int, int, int);      
      int IsRoomBeingUsed (char *, char *);
      int RoomSize (char *);      

   protected:
      void CreateTempFile (void);
      char *ReadBackup (int linenum);
      void CopyTempFile (void);
      void WaitonLock (void);

   private:
      char buf[BLOCKSIZE+1], filename[101];
      char backup[110], locfile[110];
      int  fd;
      FILE *fp, *tempfp;
};

#endif 

