// Filename: Login.cpp
// Description: Definition of Login class

#include "Login.h"

// Login constructor definition
Login::Login(Socket * socket)
{
  socket_object=socket;
  retries=0;
  logged=0;
  username[0]='\0';
}

// Definition of enter() function to enter system
int Login::enter(char login [])
{
  // Local variables
  FileSys filesystem_object;
  char filecontents[BLOCKSIZE+1];
  char name[9];
  int flag=0;

  // Check if already logged in
  if (logged==2)
  {
    socket_object->Write("500 Error.  Please log out first. \n");

    return 0;
  }

  // Open contents
  filesystem_object.OpenFile("registry.dat");  
 
  // Find matching username in registry.dat file
  for (int line=1;;line++)
  {
    strcpy(filecontents,filesystem_object.ReadFile(line));

    // Exit for loop at end of list, and set flag
    if (filecontents[0]=='\0')
    {
      retries++;
      break;
    }
    
    // Get username field
    strcpy(name,strtok(filecontents," "));

    // If have a valid username , exit for loop
    if (strcmp(name,login)==0)
    {
      // Copy username to state variable
      strcpy(username,login);

      // Signal that username is valid
      flag=1;
      logged=1; 

      // Reset retries
      retries=0;

      break;
    }
  }

  // Close file
  filesystem_object.CloseFile();

  // Send good login response
  if (flag==1)
  {
    socket_object->Write("100 OK Hello ");
    socket_object->Write(username);
    socket_object->Write(" please enter your password. \n");
  }
  else
  // Else send bad login response
  {
    switch (retries)
    {
    case 1:
      socket_object->Write("300 Invalid Username.  Failed attempt #1. \n");
      break;
    case 2:
      socket_object->Write("301 Invalid Username.  Failed attempt #2. \n");
      break;
    case 3:
      socket_object->Write(
        "302 Invalid Username.  *** Security alert ***  Closing connection. \n");
      return -1;
      break;
    }
  }
  
  return 0;
}

// Definition of password() function to enter system
int Login::password(char passwd [])
{
  // Local variables
  FileSys filesystem_object;
  char filecontents[BLOCKSIZE+1];
  char name[9];
  char pass[17];
  int flag=0;
  char encryptedpasswd[17];

  // Check if no valid username
  if (logged==0)
  {
    socket_object->Write("501 Error.  Please log in first. \n");

    return 0;
  }
  // Check if already logged into system
  else if (logged==2)
  {
    socket_object->Write("500 Error.  Please log out first. \n");

    return 0;
  }
     
  // Open contents
  filesystem_object.OpenFile("registry.dat");  
 
  // Find matching username in registry.dat file
  for (int line=1;;line++)
  {
    strcpy(filecontents,filesystem_object.ReadFile(line));

    // Exit for loop at end of list, and set flag
    if (filecontents[0]=='\0')
    {
      retries++;
      flag=1;     
      break;
    }
    
    // Encrypt password
    strcpy(encryptedpasswd,passwd); //=crypt(passwd,"69");
    
    // Get username field
    strcpy(name,strtok(filecontents," "));
    strcpy(pass,strtok(NULL,"\0"));
    // If have a valid username and valid password , exit for loop
    if (strcmp(name,username)==0 && strcmp(pass,encryptedpasswd)==0)
    {
      // Change logged state variable
      logged=2;

      // Reset retries
      retries=0;

      break;
    }
  }

  // Close file
  filesystem_object.CloseFile();

  // Send good login response
  if (flag==0)
  {
    socket_object->Write("100 OK Welcome back ");
    socket_object->Write(username);
    socket_object->Write(".  Please request now. \n");
  }
  else
  // Else send bad login response
  {
    switch (retries)
    {
    case 1:
      socket_object->Write("304 Invalid.  Failed attempt #1. \n");
      break;
    case 2:
      socket_object->Write("305 Invalid.  Failed attempt #2. \n");
      break;
    case 3:
      socket_object->Write("306 Invalid.  *** Security alert ***  Closing connection. \n");
      return -1;
      break;
    }
  }
  
  return 0;
}

// The newuser() member function definition
int Login::newuser(char username[], char passwd[])
{
  // Local variables
  Admin admin_object(socket_object);
  int status;

  // Check if already logged in
  if (logged==2)
  {
    socket_object->Write("500 Error.  Please log out first. \n");
    return 0;
  }
  
  // Add to username and password to registry.dat
  // Create user's directory
  status=admin_object.internal_add(username,passwd);

  // Check to see if username is unique
  if (status==0)
  {
    socket_object->Write("100 OK ");
    socket_object->Write(username);
    socket_object->Write(" account has been created.  Thank you. ");
    socket_object->Write("You are now logged in. \n");
  }
  else
  // Username is not unique.
  {
    // Send error response
    socket_object->Write("303 Invalid. ");
    socket_object->Write(username);
    socket_object->Write(" already exists!  ");
    socket_object->Write("Please choose another username. \n");
    return -1; 
  }

  return 0;
}

// Definition of changepasswd() member function
int Login::changepassword(char oldpasswd[], char newpasswd[])
{
  // Local variables
  FileSys filesystem_object;
  Admin admin_object(socket_object);
  char filecontents[BLOCKSIZE+1];
  char name[9];
  char pass[17];
  int flag=0;
  char encryptedpasswd[17];

  // Check if user is currently logged in
  if (logged!=2)
  {
    socket_object->Write("501 Error.  Please log in first. \n");
    return 0;
  }
  
  // Open contents
  filesystem_object.OpenFile("registry.dat");
 
  // Find matching username in registry.dat file
  for (int line=1;;line++)
  {
    strcpy(filecontents,filesystem_object.ReadFile(line));

    // Exit for loop at end of list
    if (filecontents[0]=='\0')
      break;

    strcpy(encryptedpasswd,oldpasswd); //=crypt(oldpasswd,"69");
    
    // Get username field
    strcpy(name,strtok(filecontents," "));
    strcpy(pass,strtok(NULL,"\0"));

    // If have a valid username and valid password , exit for loop
    if (strcmp(name,username)==0 && strcmp(pass,encryptedpasswd)==0)
    {
      flag=1;
      break;
    }
  }
  
  // Close file
  filesystem_object.CloseFile();

  // If old password does not match, exit
  if (flag==0)
  {
    socket_object->Write("307 Invalid.  Failed attempt.  ");
    socket_object->Write("Old password incorrect. \n");
    return 0;
  }

  // Old password matches, so execute password change    
  admin_object.internal_changepasswd(username,newpasswd);

  // Send response
  socket_object->Write("100 OK Password changed. \n");

  return 0;
}


// Definition of logout() member function
int Login::logout()
{
  // Make sure user is logged in inorder to log out
  if (logged!=2)
  {
    socket_object->Write("501 Error.  Please log in first. \n");
    return 0;
  }
  
  // Send response
  socket_object->Write("100 OK Goodbye ");
  socket_object->Write(username);
  socket_object->Write(" \n");

  return 0;
}

// Definition of help() member function
int Login::help()
{
  socket_object->Write("100 OK I will help you.  Here is a list of commands. ");
  socket_object->Write("--- LOGIN USER<Username> ");
  socket_object->Write("--- LOGIN ROOT");
  socket_object->Write("--- LOGIN NEWUSER <Username> <Password> ");
  socket_object->Write("--- LOGIN PASSWORD <Password> ");
  socket_object->Write("--- USER CHANGEPASSWORD <OldPassword> <NewPassword> ");
  socket_object->Write("--- USER LOGOUT ");
  socket_object->Write("--- USER HELP ");
  socket_object->Write("--- ADMIN DELETE <Username> ");
  socket_object->Write("--- ADMIN ADD <Username> <Password> ");
  socket_object->Write("--- ADMIN CHANGEPASSWORD <Username> <NewPassword> ");
  socket_object->Write("--- REQUEST QUICKHOURINFO <EventId> ");
  socket_object->Write("--- REQUEST DETAILEDHOURINFO <EventId> ");
  socket_object->Write("--- REQUEST INFODAY <Username> <Year> <Month> <Day> ");
  socket_object->Write("--- REQUEST CALENDAR <Year> <Month> ");
  socket_object->Write("--- REQUEST INFOMONTH <Username> <Year> <Month> ");
  socket_object->Write("--- REQUEST USERINFO <Username> <FieldVal> ");
  socket_object->Write("--- REQUEST NOTIFICATION ");
  socket_object->Write("--- REQUEST NOTES <Username> <Year> <Month> <Day> ");
  socket_object->Write("--- REQUEST TODOLIST <Username> <Year> <Month> <Day> ");
  socket_object->Write("--- REQUEST USERLIST ");
  socket_object->Write("--- REQUEST PREFS ");
  socket_object->Write("--- SET USERINFO <Username> <FieldVal> |<NewVal>| ");
  socket_object->Write("--- SET TODOLIST <Username> <Year> <Month> <Day> |<NewVal>| ");
  socket_object->Write("--- SET NOTES <Username> <Year> <Month> <Day> |<NewVal>| ");
  socket_object->Write("--- SET PREFS |<big block of client-definable text>| ");
  socket_object->Write("--- CREATE PEREVENT <EventId> <Len> |<QuickDes>|");
  socket_object->Write("|<DetailDes>| <Room> <NumInRoom> ");
  socket_object->Write("--- CREATE MEETEVENT <EventId> <Len> |<QuickDes>| ");
  socket_object->Write("|<DetailDes>| <Room> <NumInRoom> <Num> ");
  socket_object->Write("[<Username> ... <Username>] ");
  socket_object->Write("--- CREATE RESPONSE <Type> <EventId> |<Description>| ");
  socket_object->Write("--- MODIFY PEREVENT <OldEventId> <EventId> <Len> ");
  socket_object->Write("|<QuickDes>| |<DetailDes>| <Room> ");
  socket_object->Write("--- MODIFY MEETEVENT <OldEventId> <EventId> <Len> ");
  socket_object->Write("|<QuickDes>| |<DetailDes>| <Room> <Num> <Username> ");
  socket_object->Write(" ... <Username> ");
  socket_object->Write("--- DELETE PEREVENT <EventId> ");
  socket_object->Write("--- DELETE MEETEVENT <EventId> ");
  socket_object->Write("--- DELETE NOTIFICATION <From> <Type> <EventId> ");
  socket_object->Write("<Description> ");
  socket_object->Write("--- FIND NEWMEET <Year1> <Month1> <Day1> <Year2> ");
  socket_object->Write("<Month2> <Day2> <T1> <T2> <Len> <Num> ");
  socket_object->Write("[<Username> ... <Username>] ");
  socket_object->Write("---  Plus many others \n");

  return 0;
}
