#include "Parser.h"

Parser::Parser(Socket *in)
{
   comm=in;
   strcpy(loginname, "");
   log = new Login(in);
   adm = new Admin(in);
   req = new Request(in);
   per = new PerScheduler(in, "");
   grp = new GrpScheduler(in, ""); 
}

Parser::~Parser()
{
   delete log;
   delete adm;
   delete req;
   delete per;
   delete grp;
}

int Parser::Checker(int log, int typ)
{
if (log==0 && typ==0)
   {
   comm->Write("501 Error.  Please log in first. \n");
   return ERROR;
   }

if (log==1 && typ==0)
   {
   comm->Write("502 Error.  Please execute LOGIN PASSWORD command first \n");
   return ERROR;
   }

if (typ==1 && log==2)
   {
   comm->Write("500 Error.  Please log out first. \n");
   return ERROR;
   }

if (typ==3)
   {
   comm->Write("504 Error. Administrator Access Only! \n");
   return ERROR; 
   }
 
return OK;
}

void Parser::ParseRequest (char *input)
{
   char par4[9], par5[17], par6[17];
   char eid[23],year[5], day[3], month[3]; 
   char *tok;
   int ret;

   grp->SetName(loginname);
   per->SetName(loginname);

   strcpy(parsestr,input);
   strcpy(par1,strtok(input," "));
   if ((tok=strtok(NULL," "))==NULL)
      {
      comm->Write("Error: Unrecognized command \n");
      return; 
      }
   strcpy(par2,tok);
  
   //LOGIN Series commands//
   if ((strncmp(par1, "LOGIN", 5))==0)
      {
      ret=Checker(log->logged, 1);
      if (ret==ERROR)
         return;  

      if ((strncmp(par2, "USER", 4))==0)
         { 
         strcpy(par3,strtok(NULL,"\r")); //get username//
         ret=log->enter(par3);
         if (ret==ERROR)
            exit(1);         
         }

      else if ((strncmp(par2, "ROOT", 4))==0)
         {
         ret=log->enter("root");
         if (ret==ERROR)
            exit(0);         
         }

      else if ((strncmp(par2, "PASSWORD", 8))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get password//
         ret=log->password(par3);
         if (ret==ERROR)
            exit(0);              
         if (log->logged==2)
            strcpy(loginname,log->username);
         }

      else if ((strncmp(par2, "NEWUSER", 7))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get 2 parameters//
         strcpy(par4,strtok(par3," ")); //get username//
         strcpy(par5,strtok(NULL," ")); //get password//
         ret=log->newuser(par4,par5); 
         if (ret==OK)
            {
            strcpy(loginname,par3);
            log->logged=2;
            }     
         }

      else 
         comm->Write("Error: Unrecognized command \n");
      }

   //USER Series commands//
   else if ((strncmp(par1, "USER", 4))==0)
      { 
      ret=Checker(log->logged, 0);
      if (ret==ERROR)
         return;
 
      if ((strncmp(par2, "CHANGEPASSWORD", 14))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get 2 parameters//
         strcpy(par5,strtok(par3," ")); //get old password //
         strcpy(par6,strtok(NULL," ")); //get new password//
         log->changepassword(par5,par6); 
         }

      else if ((strncmp(par2, "LOGOUT", 6))==0)
         {
         log->logout();
         exit(0); 
         }

      else if ((strncmp(par2, "HELP", 4))==0)
         {
         log->help();
         }
 
      else
         comm->Write("Error: Unrecognized command \n");
      }

   //ADMIN Series commands//
   else if ((strncmp(par1, "ADMIN", 5))==0)
      {
      ret=Checker(log->logged, 0);
      if (ret==ERROR)
         return;
      
      if ((strcmp("root",loginname))!=0)
         ret=Checker(log->logged, 3);
      if (ret==ERROR)
         return;

      else if ((strncmp(par2, "DELETEROOM", 10))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get room//
         adm->DeleteRoom(par3);
         }

      if ((strncmp(par2, "DELETE", 6))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get username//
         adm->Delete(par3);
         }

      else if ((strncmp(par2, "CREATEROOM", 7))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get 2 parameters//
         strcpy(par4,strtok(par3," ")); //get room//
         strcpy(par5,strtok(NULL," ")); //get capacity//
         adm->AddRoom(par4,par5); 
         }

      else if ((strncmp(par2, "ADD", 3))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get 2 parameters//
         strcpy(par4,strtok(par3," ")); //get username//
         strcpy(par5,strtok(NULL," ")); //get password//
         ret=adm->add(par4,par5); 
         if (ret==OK)
            {
            strcpy(loginname,par3);        
            log->logged=2;
            }
         }

      else if ((strncmp(par2, "CHANGEPASSWORD", 14))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get 2 parameters//
         strcpy(par4,strtok(par3," ")); //get username //
         strcpy(par6,strtok(NULL," ")); //get new password//
         adm->changepasswd(par4,par6); 
         }

      else
         comm->Write("Error: Unrecognized command \n");
      }

    //REQUEST Series commands//
   else if ((strncmp(par1, "REQUEST", 7))==0)
      {
      ret=Checker(log->logged, 0);
      if (ret==ERROR)
         return;

      if ((strncmp(par2, "QUICKHOURINFO",13 ))==0)
         {
         strcpy(eid,strtok(NULL,"\r")); //get eid//
         req->quickhourinfo(eid);
         }

      else if ((strncmp(par2, "DETAILEDOURINFO", 15))==0)
         {
         strcpy(eid,strtok(NULL,"\r")); //get eid//
         req->detailedhourinfo(loginname,eid);
         }

      else if ((strncmp(par2, "INFODAY", 7))==0)
         {
         strcpy(par4,strtok(NULL," ")); //get username//
         strcpy(year,strtok(NULL," ")); //get year//
         strcpy(month,strtok(NULL," ")); //get month//
         strcpy(day,strtok(NULL,"\r")); //get day//
         req->infoday(par4,year,month,day);
         }

      else if ((strncmp(par2, "CALENDAR", 8))==0)
         {
         strcpy(year,strtok(NULL," ")); //get year//
         strcpy(month,strtok(NULL,"\r")); //get month//
         req->calendar(year,month);
         }

      else if ((strncmp(par2, "INFOMONTH", 9))==0)
         {
         strcpy(par4,strtok(NULL," ")); //get username//
         strcpy(year,strtok(NULL," ")); //get year//
         strcpy(month,strtok(NULL,"\r")); //get month//
         req->infomonth(par4,year,month);
         }

      else if ((strncmp(par2, "USERINFO", 8))==0)
         {
         strcpy(par4,strtok(NULL," ")); //get username//
         strcpy(par3,strtok(NULL,"\r")); //get field//
         req->userinfo(par4,par3);
         }

      else if ((strncmp(par2, "NOTIFICATION", 12))==0)
         {
         req->notification(loginname);
         }

      else if ((strncmp(par2, "NOTES", 5))==0)
         {
         strcpy(year,strtok(NULL," ")); //get year//
         strcpy(month,strtok(NULL," ")); //get month//
         strcpy(day,strtok(NULL,"\r")); //get day//
         req->notes(loginname,year,month,day);
         }

      else if ((strncmp(par2, "TODOLIST", 8))==0)
         {
         strcpy(year,strtok(NULL," ")); //get year//
         strcpy(month,strtok(NULL," ")); //get month//
         strcpy(day,strtok(NULL,"\r")); //get day//
         req->todolist(loginname,year,month,day);
         }

      else if ((strncmp(par2, "USERLIST", 8))==0)
         {
         req->userlist();
         }

      else if ((strncmp(par2, "ROOMLIST", 8))==0)
         {
         req->roomlist();
         }

      else if ((strncmp(par2, "PREFS", 5))==0)
         {
         req->prefs(loginname);
         }

      else
         comm->Write("Error: Unrecognized command \n");
      }

//call class to set username in scheduler//
   //SET Series commands//
   else if ((strncmp(par1, "SET", 3))==0)
      { 
      ret=Checker(log->logged, 0);
      if (ret==ERROR)
         return;
     
      if ((strncmp(par2, "USERINFO", 8))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get rest//
         per->SetUserInfo(par3);
         }

      else if ((strncmp(par2, "TODOLIST", 8))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get rest//
         per->SetToDoList(par3);
         }

      else if ((strncmp(par2, "NOTES", 5))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get rest//
         per->SetNotes(par3);
         }

      else if ((strncmp(par2, "PREFS", 5))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get rest//
         per->SetPrefs(par3);
         }

      else
         comm->Write("Error: Unrecognized command \n");
      }

   //CREATE Series commands//
   else if ((strncmp(par1, "CREATE", 6))==0)
      {
      ret=Checker(log->logged, 0);
      if (ret==ERROR)
         return;

      if ((strncmp(par2, "PEREVENT", 8))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get rest//
         per->CreatePerEvent(par3);
         }

      else if ((strncmp(par2, "MEETEVENT", 9))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get rest//
         grp->CreateMeetEvent(par3);
         }

      else if ((strncmp(par2, "RESPONSE", 8))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get rest//
         grp->CreateResponse(par3);
         }

      else if ((strncmp(par2, "AUTOMEET", 8))==0)
         {
        /* FILL HERE WHEN DONE */
         }

      else
         comm->Write("Error: Unrecognized command \n");
      }

   //MODIFY Series commands//
   else if ((strncmp(par1, "MODIFY", 6))==0)
      {
      ret=Checker(log->logged, 0);
      if (ret==ERROR)
         return;
 
      if ((strncmp(par2, "PEREVENT", 8))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get rest//
         per->ModifyPerEvent(par3);
         }

      else if ((strncmp(par2, "MEETEVENT", 9))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get rest//
         grp->ModifyMeetEvent(par3);
         }
    
      else
         comm->Write("Error: Unrecognized command \n");
      }

   //DELETE Series commands//
   else if ((strncmp(par1, "DELETE", 6))==0)
      {  
      ret=Checker(log->logged, 0);
      if (ret==ERROR)
         return;

      if ((strncmp(par2, "PEREVENT", 8))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get rest//
         per->DeletePerEvent(par3);
         }

      else if ((strncmp(par2, "MEETEVENT", 9))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get rest//
         grp->DeleteMeetEvent(par3);
         }

      else if ((strncmp(par2, "NOTIFICATION", 9))==0)
         {
         strcpy(par3,strtok(NULL,"\r")); //get rest//
         grp->DeleteNotification(par3);
         }

      else
         comm->Write("Error: Unrecognized command \n");
       }

   else
       comm->Write("Error: Unrecognized command \n");
}
