// Filename: Request.cpp
// Description: Definition of REQUEST class

#include "Request.h"

// Definition of the constructor
Request::Request(Socket * socket)
{
  // Assign parameter value to private variable
  socket_object = socket;
}

// Definition of the destructor
Request::~Request()
{
}

// Definition of quickhourinfo member function
int Request::quickhourinfo(char eventid[])
{
  // Local variables
  char response [BLOCKSIZE+1];
  int status = 0;
  
  // Initialization
  response[0]='\0';

  // Get information
  status = internal_quickhourinfo(eventid,response);

  // If no event for that hour
  if (status!=0)
  {
    socket_object->Write("201 No event exists. \n");
    return 0;
  }
  
  // Send response
  socket_object->Write("100 OK ");
  socket_object->Write(response);
  socket_object->Write("\n");
  
  return 0;
}

// Definition of detailedhourinfo member function
int Request::detailedhourinfo(char username[], char eventid[]) 
{
  // Local variables
  EventId eventid_object;
  FileContents filecontents_object;
  char * filecontents;
  int status;
    
  // Call object to convert eventid
  eventid_object.parse(eventid);

  // Open file
  filesystem_object.OpenEventFile(eventid_object.stripped);

  // Read file contents
  filecontents=filesystem_object.ReadFile(eventid_object.hour_int+1);

  // Close file
  filesystem_object.CloseFile();

  // Check if file contents are empty
  if (filecontents[0]=='\0' || filecontents[0]=='-')
  {
    socket_object->Write("201 No event exists. \n");
    return 0;
  }
  
  // Parse file contents and extract important info
  status=filecontents_object.parse(filecontents);

  // Check to see if the user is the creator of the event or one of the
  // invitees of the event
  if (strcmp (filecontents_object.creator,username)==0 ||
      filecontents_object.aninvitee(username)==1)
  {
    // Send ok response
    socket_object->Write("100 OK ");
    socket_object->Write(filecontents);
    socket_object->Write(" \n");
  }
  // If the user is not the creator or an invitee, send error message
  else
  {
    // Send error response
    socket_object->Write("505 Error.  Not authorized to look at ");
    socket_object->Write(filecontents_object.creator);
    socket_object->Write("'s schedule. \n");
  }
  
  return 0;
}

// Definition of infoday member function
int Request::infoday(char username[], char year[], char month[], char day[])
{
  // Local variables
  EventId eventid_object;
  char response [BLOCKSIZE+1];
  int status;
  
  // Initialization
  response[0]='\0';

  // Send beginning of response
  socket_object->Write("100 OK ");
  
  // Get the quick description for hours 0 to 23  
  for (int hour=0;hour<24;hour++)
  {
    // Make a valid eventid
    eventid_object.hour_int=hour;
    eventid_object.combine(username,year,month,day);

    // Try to write response
    status=internal_quickhourinfo(eventid_object.full,response);
 
    // If no event for that hour
    if (status!=0)
    {
      // Send no event entry response
      socket_object->Write("-1 -1 -1 ");
    }

    // Send part of response
    socket_object->Write(response);
    
    // Reset response string
    response[0]='\0';
  }

  // Send end of response
  socket_object->Write(" \n");
  
  return 0;
}

// Definition of calendar member function
int Request::calendar(char year[], char month[])
{
  // Local variables
  Calendar calendar_object;
  
  // Initialization
  calendar_object.process(year,month);

  // Send response
  socket_object->Write("100 OK ");
  socket_object->Write(calendar_object.firstdayofweek_char);
  socket_object->Write(" ");
  socket_object->Write(calendar_object.lastdayofweek_char);
  socket_object->Write(" \n");
 
  return 0;
}

// Definition of infomonth member function
int Request::infomonth(char username[], char year[], char month[])
{
  // Local variables
  EventId eventid_object;
  Calendar calendar_object;
  char day_char [3];
  int day_tmp;
  char result;
  char response[3];
      
  // Process the year and month
  calendar_object.process(year,month);

  // Send beginning of response
  socket_object->Write("100 OK ");
  
  // Determine if appointments scheduled on each day of the month
  for (int day=1;day<=calendar_object.lastdateofmonth;day++)
  {
    // Convert int to ascii
    if (day<10)
    {
      day_char[0]=day+48;
      day_char[1]='\0';
    }
    else if (day<20)
    {
      day_char[0]=1+48;
      day_tmp=day-10;
      day_char[1]=day_tmp+48;
      day_char[2]='\0';
    }
    else if (day<30)
    {
      day_char[0]=2+48;
      day_tmp=day-20;
      day_char[1]=day_tmp+48;
      day_char[2]='\0';
    }
    else
    {
      day_char[0]=3+48;
      day_tmp=day-30;
      day_char[1]=day_tmp+48;
      day_char[2]='\0';
    }

    // Get result for day
    result=internal_infoday(username,year,month,day_char);

    // Send response for day
    response[0]=result;
    response[1]=' ';
    response[2]='\0';
    socket_object->Write(response);
    socket_object->Write(" ");
  }

  // Send end of response
  socket_object->Write("\n");
  
  return 0;
}

// Definition of userinfo member function
int Request::userinfo(char username[], char field[])
{
  // Local variables
  FileSys filesystem_object;
  char filename [BLOCKSIZE+1];
  char filecontents[BLOCKSIZE+1];
  int fieldvalue_int;

  // Construct filename
  strcpy(filename,username);
  strcat(filename,"/userinfo.dat");

  // Open contents
  filesystem_object.OpenUserInfoFile(filename);  

  // Send beginning of response
  socket_object->Write("100 OK ");
  
  // Requested all fields to be returned
  if (field[0]=='0')
  {
    for (int line=1;line<=7;line++)
    {
      strcpy(filecontents,filesystem_object.ReadFile(line));
      socket_object->Write(filecontents);
      socket_object->Write(" ");
    }
  }
  else
  // Requested a particular field
  {
    // Convert ASCII to integer
    fieldvalue_int=atoi(field);
    
    // Send response
    strcpy(filecontents,filesystem_object.ReadFile(fieldvalue_int));
    socket_object->Write(filecontents);
    socket_object->Write(" ");
  }
  
  // Close file
  filesystem_object.CloseFile();

  // Send end of response
  socket_object->Write(" \n");

  return 0;
}

// Definition of notification member function
int Request::notification(char username[])
{
  // Local variables
  FileSys filesystem_object;
  char filename [BLOCKSIZE+1];
  char filecontents[BLOCKSIZE+1];

  // Construct filename
  strcpy(filename,username);
  strcat(filename,"/notify.dat");

  // Open contents
  filesystem_object.OpenFile(filename);

  // Check if there is at least one notification
  strcpy(filecontents,filesystem_object.ReadFile(1));
  if (filecontents[0]=='\0' || filecontents[0]=='-')
  {
    socket_object->Write("202 No new notifications yet. \n");

    // Close file
    filesystem_object.CloseFile();

    return 0;
  }

  // Send beginning of response
  socket_object->Write("100 OK ");

  // Read contents
  for (int line=1;;line++)
  {
    strcpy(filecontents,filesystem_object.ReadFile(line));
    
    // Exit for loop when no more entries left to read
    if (filecontents[0]=='\0' || filecontents[0]=='-')
      break;

    // Send contents to socket
    socket_object->Write(filecontents);
    socket_object->Write(" ");
  }

  // Close file
  filesystem_object.CloseFile();

  // Send end of response
  socket_object->Write(" \n");

  return 0;
}

// Definition of notes member function
int Request::notes(char username[], char year[], char month[], char day[])
{
  // Local variables
  FileSys filesystem_object;
  FileContents filecontents_object;
  char filename [BLOCKSIZE+1];
  char filecontents[BLOCKSIZE+1];
  
  // Construct filename
  strcpy(filename,"");
  sprintf(filename,"%s/%s.%s.%s.notes.dat",username, year, month, day);

  // Open contents
  filesystem_object.OpenFile(filename);  
  
  strcpy(filecontents,filesystem_object.ReadFile(1));
  filesystem_object.CloseFile();

  // Send response
  socket_object->Write("100 OK ");
  socket_object->Write(filecontents);
  socket_object->Write(" \n");

  return 0;
}  

// Definition of todolist member function
int Request::todolist(char username[], char year[], char month[], char day[])
{
  // Local variables
  FileSys filesystem_object;
  FileContents filecontents_object;
  char filename [BLOCKSIZE+1];
  char filecontents[BLOCKSIZE+1];
  
  // Construct filename
  strcpy(filename,"");
  sprintf(filename,"%s/%s.%s.%s.todo.dat",username,year,month,day);

  // Open contents
  filesystem_object.OpenFile(filename);  
  
  strcpy(filecontents,filesystem_object.ReadFile(1));
  filesystem_object.CloseFile();

  // Send response
  socket_object->Write("100 OK ");
  socket_object->Write(filecontents);
  socket_object->Write(" \n");

  return 0;
}  

// Definition of userlist member function
int Request::userlist()
{
  // Local variables
  FileSys filesystem_object;
  char filecontents[BLOCKSIZE+1];
  
  // Open contents
  filesystem_object.OpenFile("registry.dat");  

  // Check if there is at least one user in list
  strcpy(filecontents,filesystem_object.ReadFile(1));
  if (filecontents[0]=='\0' || filecontents[0]=='-')
  {
    socket_object->Write("203 No users registered yet --- registry empty. \n");

    // Close file
    filesystem_object.CloseFile();

    return 0;
  }
  
  // Send beginning of response
  socket_object->Write("100 OK ");

  // Read contents
  for (int line=1;;line++)
  {
    strcpy(filecontents,filesystem_object.ReadFile(line));

    // Exit for loop at end of list
    if (filecontents[0]=='\0')
      break;
    
    // Send username but not password
    socket_object->Write(strtok(filecontents," "));
    socket_object->Write(" ");
  }

  // Close file
  filesystem_object.CloseFile();

  // Send end of response
  socket_object->Write(" \n");
  
  return 0;
}  

// Definition of roomlist member function
int Request::roomlist()
{
  // Local variables
  FileSys filesystem_object;
  FileContents filecontents_object;
  char filecontents[BLOCKSIZE+1];

  // Open contents
  filesystem_object.OpenFile("rooms/listing.dat");  

  // If rooms exist
  strcpy(filecontents,filesystem_object.ReadFile(1));
  if (filecontents[0]=='\0' || filecontents[0]=='-')
  {
    socket_object->Write("339 No rooms defined yet. \n");

    // Close file
    filesystem_object.CloseFile();

    return 0;
  }

  // Send beginning of response
  socket_object->Write("100 OK ");
  
  // Read contents
  for (int line=1;;line++)
  {
    strcpy(filecontents,filesystem_object.ReadFile(line));
    
    // Exit for loop when at end of list
    if (filecontents[0]=='\0' || filecontents[0]=='-')
      break;

    socket_object->Write(filecontents);
    socket_object->Write(" ");
  }

  // Close file
  filesystem_object.CloseFile();

  // Send end of response
  socket_object->Write(" \n");

  return 0;
}  

// Definition of prefs member function
int Request::prefs(char username[])
{
  // Local variables
  FileSys filesystem_object;
  FileContents filecontents_object;
  char filename [BLOCKSIZE+1];
  char filecontents[BLOCKSIZE+1];

  // Construct filename
  strcpy(filename,username);
  strcat(filename,"/pref.dat");

  // Open contents
  filesystem_object.OpenFile(filename);  
  
  strcpy(filecontents,filesystem_object.ReadFile(1));
  // If file contents are empty
  if (filecontents[0]=='\0' || filecontents[0]=='-')
  {
    socket_object->Write("205 Prefs are empty. \n");

    // Close file
    filesystem_object.CloseFile();

    return 0;
  }

  filesystem_object.CloseFile();

  // Send response
  socket_object->Write("100 OK ");
  socket_object->Write(filecontents);
  socket_object->Write(" \n");

  return 0;
}  

// Definition of internal_quickhourinfo member function
int Request::internal_quickhourinfo(char eventid[], char response[])
{
  // Local variables
  EventId eventid_object;
  FileContents filecontents_object;
  char filecontents[BLOCKSIZE+1];
  int status = 0;
  
  // Initialization
  response[0]='\0';
  
  // Call object to convert eventid
  eventid_object.parse(eventid);

  // Open file
  filesystem_object.OpenEventFile(eventid_object.stripped);
   
  // Read file contents
  strcpy(filecontents,filesystem_object.ReadFile(eventid_object.hour_int+1));

  // Close file
  filesystem_object.CloseFile();

  // Parse file contents and extract important info
  status=filecontents_object.parse(filecontents);

  // If the file contents were not empty
  if (status==0)
  {
    // Construct response
    strcat(response,filecontents_object.creator);
    strcat(response," ");
    strcat(response,filecontents_object.length_char);
    strcat(response," ");
    strcat(response,filecontents_object.quickdesc);
    strcat(response," ");
  }
  else
    // Else file contents were empty
    return -1;    

  return 0;
}

// Definition of internal_infoday member function
char Request::internal_infoday(char username[], char year[], char month[], 
                               char day[])
{
  // Local variables
  EventId eventid_object;
  char response[BLOCKSIZE+1];
  int status = 0;
    
  // Initialization
  response[0]='\0';
  
  // Get the quick description for hours 0 to 23  
  for (int hour=1;hour<=24;hour++)
  {
    // Make a valid eventid
    eventid_object.hour_int=hour;
    eventid_object.combine(username,year,month,day);

    // Try to get a response
    status=internal_quickhourinfo(eventid_object.full,response);
 
    // If there is an event for that hour, return '1'
    if (status==0)
      return '1';
        
    // Reset response string
    response[0]='\0';
  }

  // Report back to calling function that there is nothing scheduled for that
  // day
  return '0';
}
