// David Tam
// Filename: Request.h
// Description: Header for REQUEST class
// Created Mon. Oct 19, 1998.
// Modified Tues Oct 20; Wed Oct 21; Fri Oct 23; Sat Oct 24; Fri Oct 30;

// Include this header file only if not included previously
#ifndef REQUEST_H
#define REQUEST_H

// Include the socket & file system class declarations
#include <string.h>   /* Needed for string copy functions */
#include <stdlib.h>   /* Needed for ascii to integer conversion */
#include <iostream.h>   /* Needed for testing purposes */
#include "Socket.h"
#include "Filesys.h"
#include "RequestInternal.h"   /* Internal support code */

// Max size of a line in each file
#ifndef BLOCKSIZE
#define BLOCKSIZE 4096
#endif

// Max length of username
#ifndef USERNAME_LENGTH
#define USERNAME_LENGTH 8
#endif

// Request class declaration
class Request {
  public:
  Request(Socket * socket);
  ~Request();
  int quickhourinfo(char eventid[]);

  // In the function below, the username parameter is the name of the user
  // making the request.  This is needed because only the creator of the event
  // or the invitees of the event may see the detailed hour info   
  int detailedhourinfo(char username[], char eventid[]);

  int infoday(char username[], char year[], char month[], char day[]);
  int calendar(char year[], char month[]);
  int infomonth(char username[], char year[], char month[]);
  int userinfo(char username[], char field[]);
  int notification(char username[]);
  int notes(char username[], char year[], char month[], char day[]);
  int todolist(char username[], char year[], char month[], char day[]);
  int userlist();
  int roomlist();
  int prefs(char username[]);

  private:
  Socket * socket_object;
  FileSys filesystem_object;
  int internal_quickhourinfo(char eventid[], char response[]);  
  char internal_infoday(char username[], char year[], char month[], char day[]);
};

#endif
