// David Tam
// Filename: RequestInternal.cpp
// Description: Definition of EventId and FileContents classes
// Created Wed. Oct 21, 1998.
// Modified Fri Oct 23; Sat Oct 24; Fri Oct 31

#include "RequestInternal.h"

// Definition of the EventId constructor
EventId::EventId()
{
  stripped[0]='\0';
  stripped2[0]='\0';
  full[0]='\0';
  hour_char[0]='\0';
  hour_int=0;
}

// Definition of the EventId destructor
EventId::~EventId()
{
}

// Definition of the EventId parse() member function
int EventId::parse(char eventid[])
{
  // Local variables
  char eventid_tmp[BLOCKSIZE+1];

  // Initialization
  stripped[0]='\0';
  stripped2[0]='\0';
  full[0]='\0';
  hour_char[0]='\0';
  hour_int=0;

  // Copy original into char arrays
  strcpy(full,eventid);
  strcpy(eventid_tmp,eventid);

  // Convert eventid into a valid filename
  strcpy(stripped,strtok(eventid_tmp,"."));
  strcat(stripped,"/");
  strcat(stripped,strtok(NULL,"."));
  strcat(stripped,".");
  strcat(stripped,strtok(NULL,"."));
  strcat(stripped,".");
  strcat(stripped,strtok(NULL,"."));
  strcat(stripped,".events.dat");

  // Restore temp string
  strcpy(eventid_tmp,eventid);

  // Convert eventid into a valid filename
  strcpy(stripped2,strtok(eventid_tmp,"."));
  strcat(stripped2,"/");
  strcat(stripped2,strtok(NULL,"."));
  strcat(stripped2,".");
  strcat(stripped2,strtok(NULL,"."));
  strcat(stripped2,".");
  strcat(stripped2,strtok(NULL,"."));

  // Restore temp string
  strcpy(eventid_tmp,eventid);

  // Save hour info
  strtok(eventid_tmp,".");
  strtok(NULL,".");
  strtok(NULL,".");
  strtok(NULL,".");
  strcpy(hour_char,strtok(NULL,"\0"));
  hour_int=atoi(hour_char);
  
  return 0;
}

// Definition of the EventId combine() member function
int EventId::combine(char username[], char year[], char month[], char day[])
{
  // Initialization
  stripped[0]='\0';
  full[0]='\0';

  // Combine into the form stripped[]="username/year.month.day.events.dat"
  strcpy(stripped,username);
  strcat(stripped,"/");
  strcat(stripped,year);
  strcat(stripped,".");
  strcat(stripped,month);
  strcat(stripped,".");
  strcat(stripped,day);
  strcat(stripped,".events.dat");

  // Combine into the form stripped2[]="username/year.month.day"
  strcpy(stripped2,username);
  strcat(stripped2,"/");
  strcat(stripped2,year);
  strcat(stripped2,".");
  strcat(stripped2,month);
  strcat(stripped2,".");
  strcat(stripped2,day);

  // Convert hour_int to hour_char
  if (hour_int<10)
  {
    hour_char[0]=hour_int+48;
    hour_char[1]='\0';
  }
  else if (hour_int<20)
  {
    hour_char[0]=1+48;
    hour_int=hour_int-10;
    hour_char[1]=hour_int+48;
    hour_char[2]='\0';
  }
  else
  {
    hour_char[0]=2+48;
    hour_int=hour_int-20;
    hour_char[1]=hour_int+48;
    hour_char[2]='\0';
  }
    
  // Combine into the form stripped[]="username.year.month.day.hour"
  strcpy(full,username);
  strcat(full,".");
  strcat(full,year);
  strcat(full,".");
  strcat(full,month);
  strcat(full,".");
  strcat(full,day);
  strcat(full,".");
  strcat(full,hour_char);

  return 0;
}

// Definition of the FileContents constructor
FileContents::FileContents()
{
  length_char[0]='\0';
  length_int=0;
  quickdesc[0]='\0';
  detaildesc[0]='\0';
  room[0]='\0';
  numinroom[0]='\0';
  creator[0]='\0';
  numinvitees_char[0]='\0';
  numinvitees_int=0;
  remainder[0]='\0';

}

// Definition of the FileContents destructor
FileContents::~FileContents()
{
}

// Definition of the FileContents parse() member function
int FileContents::parse(char filecontents[])
{
  // Local variables
  int unsigned currentchar;

  // Reinitialize variables to blank
  length_char[0]='\0';
  length_int=0;
  quickdesc[0]='\0';
  detaildesc[0]='\0';
  room[0]='\0';
  numinroom[0]='\0';
  creator[0]='\0';
  numinvitees_char[0]='\0';
  numinvitees_int=0;
  remainder[0]='\0';

  // Check to see if filecontents contains only a "\n");
  if (strlen(filecontents)<=1)
    return -1;
    
  // Get data for length variable
  for(currentchar=0;currentchar<strlen(filecontents);currentchar++)
  {
    // Check if at end of word
    if (filecontents[currentchar]==' ')
    {
      // Go to next word
      currentchar=currentchar+2; // Go to character after | symbol
      break;
    }

    // Add 1 char to length variable
    strncat(length_char,(const char *)&filecontents[currentchar],1);
  }
  
  // Also convert length to an integer
  length_int=atoi(length_char);

  // Get quick description
  strcpy(quickdesc,"|");   // Add beginning '|'
  for(;currentchar<strlen(filecontents);currentchar++)
  {
    // Check if at end of quick description
    if (filecontents[currentchar]=='|')
    {
      // Add ending '|'
      strcat(quickdesc,"|");
      
      // Go to next word
      currentchar=currentchar+3;

      break;
    }

     // Add 1 char to quickdesc variable
     strncat(quickdesc,(const char *)&filecontents[currentchar],1);
  }

  // Get detailed description
  strcpy(detaildesc,"|");   // Add beginning '|'
  for(;currentchar<strlen(filecontents);currentchar++)
  {
    // Check if at end of detailed description
    if (filecontents[currentchar]=='|')
    {
      // Add ending '|'
      strcat(detaildesc,"|");

      // Go to beginning of next word
      currentchar=currentchar+2;

      break;
    }

    // Add 1 char to detaildesc
    strncat(detaildesc,(const char *)&filecontents[currentchar],1);
  }

  // Get room of meeting
  for (;currentchar<strlen(filecontents);currentchar++)
  {
    // Check if at end of room name
    if (filecontents[currentchar]==' ')
    {
      // Go to beginning of next word
      currentchar++;
      break;
    }

    // Add to the room variable
    strncat(room,(const char *)&filecontents[currentchar],1);
  }  

  // Get total number in the room
  for (;currentchar<strlen(filecontents);currentchar++)
  {
    // Check if at end of total in the room
    if (filecontents[currentchar]==' ')
    {
      // Go to beginning of next word
      currentchar++;
      break;
    }

    // Add to the room variable
    strncat(numinroom,(const char *)&filecontents[currentchar],1);
  }  

  // Get creator
  for(;currentchar<strlen(filecontents);currentchar++)
  {
    // Check if at end of creator
    if (filecontents[currentchar]==' ')
    {
      // Go to beginning of next word
      currentchar=currentchar+1;
      break;
    }
    
    // Add to the creator variable
    strncat(creator,(const char *)&filecontents[currentchar],1);
  }
   
  // Get numinvitees
  for(;currentchar<strlen(filecontents);currentchar++)
  {
    // Check if at end of numinvitees
    if (filecontents[currentchar]==' ')
    {
      // Go to beginning of next word
      currentchar=currentchar+1;

      break;
    }

    // Add to the creator variable
    strncat(numinvitees_char,(const char *)&filecontents[currentchar],1);
  }
 
  // Also convert numinvitees to an integer
  numinvitees_int=atoi(numinvitees_char);

  // Keeping the usernames and their status in a different string
  strcpy(remainder,(const char *)&filecontents[currentchar]);
    
  return 0;
}

// Definition of the FileContents aninvitee() member function
int FileContents::aninvitee(char username[])
{
  if (strstr(remainder,username)!=NULL)
    return 1;
  
  // If this point is reached, username is not a invitee of the event
  return 0;
}

// Definition of the Calendar constructor
Calendar::Calendar()
{
  lastdateofmonth=0;
  firstdayofweek_int=0;
  lastdayofweek_int=0;
  firstdayofweek_char [0]='\0';
  firstdayofweek_char [1]='\0';
  lastdayofweek_char [1]='\0';
  in_year=0;
  in_month=0;  
}

// Definition of the Calendar destructor
Calendar::~Calendar()
{
}

// Definition of the Calendar process member function
int Calendar::process(char year[], char month[])
{
  // Local variables
  int y,m,d;
  
  // Convert month and year to integers
  m=atoi(month);
  y=atoi(year);
  
  // Copy to state variables
  in_month=m;
  in_year=y;
  
  // Calculate day of week for 1st day of month
  d=1;
  
  // Following eqn from http://users.aol.com/s6sj7gt/mikecal.htm
  // By:  Mike Keith  domnei@aol.com
  // Y2K compliant, 0=Sun, 6=Sat
  firstdayofweek_int=(d+=m<3?y--:y-2,23*m/9+d+4+y/4-y/100+y/400)%7;

  // Convert int to ascii
  firstdayofweek_char[0]=firstdayofweek_int+48;

  // Value of y might have been modified. Restore original value
  y=in_year;
  
  // Determine last date of the month
  switch (m)
  {
    // Jan, Mar, May, Jul, Aug, Oct, Dec
    case 1:
    case 3:
    case 5:
    case 7:
    case 8:
    case 10:
    case 12:
      lastdateofmonth=31;
      break;
    // February
    case 2:
      // Check if leap year
      // A year is a leap year if it is divisible by 4 but not by 100,
      // except that years divisible by 400 ARE leap years.
      if ((y%4==0 && y%100!=0) || y%4==0)
        // Yes, it is a leap year
        lastdateofmonth=29;        
      else
        // No, it is NOT a leap year
        lastdateofmonth=28;
      break;
    // Apr, Jun, Sept, Nov
    default:
      lastdateofmonth=30;
  }
 
  // Perform calc for last day of the month
  d=lastdateofmonth;
  lastdayofweek_int=(d+=m<3?y--:y-2,23*m/9+d+4+y/4-y/100+y/400)%7;

  // Convert int to ascii
  lastdayofweek_char[0]=lastdayofweek_int+48;

  return 0;
}
