// David Tam
// Filename: RequestInternal.h
// Description: Header for EventId and FileContents class
// Created Wed. Oct 21, 1998.
// Modified Fri Oct 23; Sat Oct 24; Fri Oct 30;

// Include this header file only if not included previously
#ifndef REQUESTINTERNAL_H
#define REQUESTINTERNAL_H

#include <string.h>   /* Needed for string copy functions */
#include <stdlib.h>   /* Needed for string to integer conversion */
#include <stdio.h>    /* Needed for the sscanf function */
#include <iostream.h> /* Needed for testing purposes */

// Max size of a line in each file
#ifndef BLOCKSIZE
#define BLOCKSIZE 4096
#endif

// Max length of username
#ifndef USERNAME_LENGTH
#define USERNAME_LENGTH 8
#endif

// Max length of room
#ifndef ROOM_LENGTH
#define ROOM_LENGTH 12
#endif

// EventId class declaration
class EventId {
  public:
  EventId();
  ~EventId();
  int parse(char eventid[]);
  int combine(char username[], char year[], char month[], char day[]);
  // Public variables
  char stripped [34];
  char stripped2 [20];
  char full [23];
  char hour_char [3];
  int hour_int;
};
// Notes for EventId class
// =======================
// When the object is created, state variables have no value.
// Use member function parse() to load object with the eventid.
// State variables stripped[] to hour_int will then be set correctly.
// stripped[] is set to username/year/month/day
//
// The combine() function is used to combine the the username, year, month, and
// day parameter into a stripped version of the eventid.
// After calling this function, it sets stripped[]=username/year/month/day
// , full[]=username.year.month.day.hour_int , and hour_char=hour_int

// FileContents class declaration
class FileContents {
  public:
  FileContents();
  ~FileContents();
  int parse(char filecontents[]);
  char * invitee(int number);
  char * attendance_char(int number);
  int attendance_int(int number);  
  int aninvitee(char username[]);

  // Public variables
  char length_char[3];
  int length_int;
  char quickdesc [BLOCKSIZE+1];
  char detaildesc [BLOCKSIZE+1];
  char room [ROOM_LENGTH+1];
  char numinroom [4];
  char creator [USERNAME_LENGTH+1];
  char numinvitees_char [3];
  int numinvitees_int;

  private:
  char remainder[BLOCKSIZE+1];
};
// Notes for FileContents class
// ============================
// When the object is created, state variables have no value.
// Use member function parse() to load object with the file contents.
// State variables length_char[] to numinvitees_int will then be set correctly.
// The normal return value is 0. If the file contents are empty and contain only
// a single carrage return <CR>, then the return value is -1.

// Functions not implemented yet
// -----------------------------
// Once the object is loaded, can use functions invitee(),attendance_char(),
// attendance_int().
// All invitees are numbered starting at 1. To retreive the username of the 3rd
// invitee, call invitee(3);
// To get the 3rd invitee's attendance status as an integer,
// call attendance_int(3)
// To get the 3rd invitee's attendance status as a string,
// call attendance_char(3)
//
// The aninvitee() function will see if username is an invitee of the event.
// It has a return value of 0 if username is not an invitee and
// a return value of 1 if the username is an invitee.

// Calendar class declaration
class Calendar {
  public:
  Calendar();
  ~Calendar();
  int process(char year[], char month[]);
  int lastdateofmonth;
  int firstdayofweek_int;
  int lastdayofweek_int;
  char firstdayofweek_char [2];
  char lastdayofweek_char [2];
  
  private:
  int in_year;
  int in_month;
};
#endif
