#include "Scheduler.h"

Scheduler::Scheduler(Socket *talk, const char *name)
{
   comm=talk;
   strcpy(username,name);
}

void Scheduler::SetName (char *in)
{
   strcpy(username,in);
}

char *Scheduler::FindRoom(int numusers, int y, int m, int d, int h, int l)
{
   return FlatFile.SearchRoomListing(numusers,y,m,d,h,l);
}

int Scheduler::CreateResponse (const char *input)
{
   char typ[2], eid[23], odesc[257], ndesc[257], user[9], len[3]; 
   char year[5], day[3], month[3], hour[3]; 
   char str[BLOCKSIZE+1];
   char *fromfile, *tb1, *tb2, *tb3, *tb4;
   unsigned int i;
   int j=0, currhour, flag=0;
   
   //Parse string to extract relevant parameters//
   strcpy(str,input);
   strcpy(typ,strtok(str, " "));   //get type 
   strcpy(eid,strtok(NULL, " "));  //get eventid
   strcpy(len,strtok(NULL, " "));  //get len
   strcpy(odesc,strtok(NULL, "|"));  //get description
   strcpy(ndesc,strtok(NULL, "\0"));  //get description

   if (atoi(typ)<0 || atoi(typ)>5)
      {
      comm->Write("324 Invalid Type. \n");
      return ERROR;
      }
 
   strcpy(str,eid);
   strcpy(user,strtok(str, ".")); //get user
   strcpy(year,strtok(NULL, ".")); 
   strcpy(month,strtok(NULL, ".")); 
   strcpy(day,strtok(NULL, ".")); 
   strcpy(hour,strtok(NULL, " ")); 
 
   //Delete message from current user notification box//
   strcpy(command,"");
   sprintf (command, "%s 0 %s |%s|",user, eid, odesc);
   sprintf(path,"%s/notify.dat",username);
   FlatFile.OpenFile(path);
   FlatFile.DeleteNotification(command);
   FlatFile.CloseFile();

   //Send notification to recipient//
   strcpy(command,"");
   sprintf (command, "%s %s %s%s",username, typ, eid, ndesc);
   sprintf(path,"%s/notify.dat",user);
   FlatFile.OpenFile(path);
   FlatFile.WriteFile(command,0,0);
   FlatFile.CloseFile();
 
   //Update Meeting Event Status bits
   strcpy(command,"");
   sprintf(path,"%s/%s.%s.%s.events.dat",user,year,month,day);
   FlatFile.OpenEventFile(path);
   fromfile=FlatFile.ReadFile(atoi(hour)+1);
   strcpy(str,fromfile);
   tb1=strtok(str," ");
   tb2=strtok(NULL,"|");
   tb3=strtok(NULL,"|");
   tb3=strtok(NULL,"|");
   tb4=strtok(NULL,"\n");

   strcpy(command,"");
   for (i=0; i<strlen(tb4); i++)
      {
      if (tb4[i]=='0' || tb4[i]=='1' || tb4[i]=='2' || tb4[i]=='3' || 
          tb4[i]=='4' || tb4[i]==' ')
         { 
         strcpy(command,"");
         j=0;
         continue; 
         }   
      
      command[j++]=tb4[i];
      command[j]='\0';
      if ((strncmp(username,command,strlen(username)))==0)
         {
         flag=1;
         tb4[i+2]=typ[0];
         break;  
         }
      }

   sprintf (command,"%s |%s| |%s|%s", tb1, tb2, tb3, tb4);
   FlatFile.WriteFile(command,atoi(hour)+1,24);
   FlatFile.CloseFile();

   //Update Personal Event Calendar//
   if (flag==1) //only if meeting exists, it may have been deleted//
      {
      sprintf(path,"%s/%s.%s.%s.events.dat",username,year,month,day);
      FlatFile.OpenEventFile(path);
      if (atoi(typ)==1)
         {
         currhour=atoi(hour);
         for (j=0; j<atoi(len); j++)   
            FlatFile.WriteFile(eid,++currhour,24); 
         }

      else if (atoi(typ)==4)
         {
         currhour=atoi(hour);
         for (j=0; j<atoi(len); j++)   
            FlatFile.WriteFile(eid,++currhour,24); 
         
         }

      FlatFile.CloseFile();
      comm->Write("100 OK Notification sent. \n");
      return OK;   
   }

   comm->Write("342 Error, meeting does not exist \n");
   return OK;
}

int Scheduler::DeleteNotification (const char *input)
{
   char typ[2], eid[23], desc[257], user[9];
   char str[BLOCKSIZE+1];

   //Parse string to extract relevant parameters//
   strcpy(str,input);
   strcpy(user,strtok(str, " "));  //get from
   strcpy(typ,strtok(NULL, " "));  //get type 
   if (atoi(typ)==0)
      {
      comm->Write("324 Invalid Type. \n");
      return OK;  
      }  

   strcpy(eid,strtok(NULL, " "));  //get eventid 
   strcpy(desc,strtok(NULL, "|")); //get description
 
   //Delete message from current user notification box//
   strcpy(command,"");
   sprintf (command, "%s %s %s |%s|",user, typ, eid, desc);
   sprintf(path,"%s/notify.dat",username);
   FlatFile.OpenFile(path);
   FlatFile.DeleteNotification(command);
   FlatFile.CloseFile();

   comm->Write("100 OK Notification deleted. \n");
   return OK;
}

int Scheduler::SetUserInfo(const char *input)
{ 
   char lin[2], desc[257], user[9];
   char str[BLOCKSIZE+1];
   int changeid;

   //Parse string to extract relevant parameters//
   strcpy(str,input);
   strcpy(user,strtok(str, " "));  //get user
   if ((strcmp(user,username))!=0)
      {
      comm->Write("506 Error. Not authorized to set ");
      comm->Write(user);
      comm->Write(" 's info. \n");
      return ERROR;
      }
   
   strcpy(lin,strtok(NULL, " "));  //get line 
   strcpy(desc,strtok(NULL, "|")); //get description
   
   changeid=atoi(lin);
   if (changeid<1 || changeid>7)
      {
      comm->Write("315 Invalid FieldVal. \n");
      return ERROR;
      }

   //Set the UserInfo//
   sprintf(path,"%s/userinfo.dat",user);
   FlatFile.OpenUserInfoFile(path);
   FlatFile.WriteFile(desc,atoi(lin),7);
   FlatFile.CloseFile();

   if (changeid==1)
      comm->Write("100 OK First Name set. \n");
   else if (changeid==2)
      comm->Write("100 OK Last Name set. \n");
   else if (changeid==3) 
      comm->Write("100 OK Address set. \n");
   else if (changeid==4) 
      comm->Write("100 OK Phone # set. \n");
   else if (changeid==5) 
      comm->Write("100 OK Fax # set. \n");
   else if (changeid==6) 
      comm->Write("100 OK Email set. \n");
   else if (changeid==7) 
      comm->Write("100 OK Comment set. \n");

  return OK;
}

int Scheduler::SetToDoList(const char *input)
{ 
   char user[9], year[5], month[3], day[3], desc[257];
   char str[BLOCKSIZE+1];

   //Parse string to extract relevant parameters//
   strcpy(str,input);
   strcpy(user,strtok(str, " "));  //get user
   if ((strcmp(user,username))!=0)
      {
      comm->Write("506 Error. Not authorized to set ");
      comm->Write(user);
      comm->Write(" 's info. \n");
      return ERROR;
      }
   
   strcpy(year,strtok(NULL, " "));  //get year
   strcpy(month,strtok(NULL, " "));  //get month
   strcpy(day,strtok(NULL, " "));  //get day
   strcpy(desc,strtok(NULL, "|")); //get description
   sprintf (command, "|%s|",desc);
   
   //Set the ToDoList//
   sprintf(path,"%s/%s.%s.%s.todo.dat",user,year,month,day);
   FlatFile.OpenFile(path);
   FlatFile.WriteFile(command,1,1);
   FlatFile.CloseFile();

   comm->Write("100 OK To Do List set. \n");
   return OK;
}

int Scheduler::SetNotes (const char *input) 
{
   char user[9], year[5], month[3], day[3], desc[257];
   char str[BLOCKSIZE+1];

   //Parse string to extract relevant parameters//
   strcpy(str,input);
   strcpy(user,strtok(str, " "));  //get user
   if ((strcmp(user,username))!=0)
      {
      comm->Write("506 Error. Not authorized to set ");
      comm->Write(user);
      comm->Write(" 's info. \n");
      return ERROR;
      }
   
   strcpy(year,strtok(NULL, " "));  //get year
   strcpy(month,strtok(NULL, " "));  //get month
   strcpy(day,strtok(NULL, " "));  //get day
   strcpy(desc,strtok(NULL, "|")); //get description
   sprintf (command, "|%s|",desc);
   
   //Set the Notes List//
   sprintf(path,"%s/%s.%s.%s.notes.dat",user,year,month,day);
   FlatFile.OpenFile(path);
   FlatFile.WriteFile(command,1,1);
   FlatFile.CloseFile();

   comm->Write("100 OK Note List set. \n");
   return OK;
}

int Scheduler::SetPrefs(const char *input)
{
   strcpy(command, input);
   
   //Set the Pref File//
   sprintf(path,"%s/pref.dat",username);
   FlatFile.OpenFile(path);
   FlatFile.WriteFile(command,1,1);
   FlatFile.CloseFile();

   comm->Write("100 OK Block of client-definable text accepted. \n"); 
   return OK;
}

